<?php
defined('Trapper') or exit('Trespasser!');

/**
 *
 * @author Hutton
 * @name Game Adder Mochi Model Class
 * @license GoLevel LLC
 */

Class SAA_Model_Game_Adder_Mochi extends SAA_Model
{
    private $_aMap = array();
    private $_aParam = array();

    /**
     * Class Constructor
     */
    public function  __construct()
    {
        $aType = SAA_Sys::getModel('game.type')->get('MochiMedia', false);

        $this->_aParam['type'] = $aType['game_type_id'];

        $this->_aMap = SAA_Sys::getModel('category.map')->getAllType($this->_aParam['type']);
    }

    /**
     * Sorts game data
     *
     * @param array $aRaw
     * @return array
     */
    public function get($aRaw)
    {
        return array
        (
            'game' => array
            (
                'type_id' => $this->_aParam['type'],
                'slug' => $this->parse()->toAlphaNumeric($aRaw['slug']),
                'px_height' => (int) $aRaw['height'],
                'px_width' => (int) $aRaw['width'],
                'scoring' => (int) $aRaw['leaderboard_enabled'],
                'time_added' => SAA_TIME
            ),
            'category' => $this->getCategory($aRaw),
            'extra' => array
            (
                'source_type' => $this->_aParam['type'],
                'source_id' => $aRaw['game_tag'],
            ),
            'scoring' => array
            (
                'scoring_type' => $this->_aParam['type'],
                'scoring_hash' => $aRaw['game_tag'],
            ),
            'tag' => $this->getTags($aRaw['tags']),
            'text' => array
            (
                'title' => $this->parse()->removeTags($aRaw['name']),
                'description' => $this->parse()->cleanTags($aRaw['description'], 'global.allowed_tags'),
                'direction' => $this->parse()->cleanTags($aRaw['instructions'], 'global.allowed_tags')
            )
        );
    }

    /**
     * Gets category data for a given game
     *
     * @param array $aRaw
     * @return array
     */
    public function getCategory($aRaw)
    {
        $aMap = array();

        $aCategories = $aRaw['category'];

        foreach($aCategories as $sCategory)
        {
            $sCategory = strtolower(trim($sCategory));

            $aMap[] = array(
                'category_name' => $sCategory,
                'map_id' => $this->_aMap[$sCategory]
            );
        }

        return $aMap;
    }

    /**
     * Gets image
     *
     * @param array $aRaw
     * @return string
     */
    public function getImg($aRaw)
    {
        return $aRaw['thumbnail_url'];
    }

    /**
     * Get maximum number of games
     *
     * @return string
     */
    public function getMax()
    {
        return $this->_aParam['max'];
    }

    /**
     * Gets a swf location
     *
     * @param array $aRaw
     * @return string
     */
    public function getSwf($aRaw)
    {
        return $aRaw['swf_url'];
    }

    /**
     * Gets the tags for a given game
     *
     * @param array $aTags
     * @return array
     */
    public function getTags($aTags)
    {
        foreach($aTags as $i=>$sTag)
        {
            $aTags[$i] = strtolower($sTag);
        }

        return $aTags;
    }

    /**
     * Determine whether or not files will be moved locally
     *
     * @return bool
     */
    public function importFiles()
    {
        return SAA_Sys::getSett('mochi.local_files');
    }

    /**
     * Gets games from source using a filter
     *
     * @param array $aParam
     * @param int $iOffset
     * @param int $iLimit
     * @return mixed
     */
    public function import($aParam, $iOffset = 50, $iLimit = 25)
    {
        $oFile = SAA_Sys::getLib('file');

        $sQuery = http_build_query(array(
            'format' => 'json',
            'limit' => $iLimit,
            'offset' => $iOffset,
        ));

        if( isset($aParam['chinese']) && $aParam['chinese'] == 'yes' )
        {
            $sQuery .= '&tag=-zh-cn';
        }

        $sUrl = 'http://www.mochimedia.com/feeds/games/'. SAA_Sys::getSett('mochi.publisher_id') .'/?'.$sQuery;

        if( !$oFile->downloadRemote($sUrl, SAA_APP_CACHE . 'adder.tmp.txt') )
        {
            return SAA_Error::add('JSON file could not be downloaded to server.');
        }

        if( !($sRaw = file_get_contents(SAA_APP_CACHE . 'adder.tmp.txt')) )
        {
            return SAA_Error::add('JSON file could not be opened.');
        }

        $aJson = json_decode($sRaw, true);

        if( is_null($aJson) )
        {
            return SAA_Error::add('JSON file could not be accessed.');
        }
        elseif( !isset($aJson['games']) || !count($aJson['games']) )
        {
            return 'complete';
        }

        $oFile->delete(SAA_APP_CACHE . 'adder.tmp.txt');

        $aGames = array();

        $iMax = (isset($aParam['max_games']) ? (int) $aParam['max_games'] : 0);

        $this->setMax($iMax);

        foreach($aJson['games'] as $aGame)
        {
            $bSkip = false;

            if( isset($aParam['max_width']) && is_numeric($aParam['max_width']) && (!isset($aGame['width']) || $aParam['max_width'] < $aGame['width']) )
            {
                $bSkip = true;
            }
            elseif( isset($aParam['max_height']) && is_numeric($aParam['max_height']) && (!isset($aGame['height']) || $aParam['max_height'] < $aGame['height']) )
            {
                $bSkip = true;
            }
            elseif( isset($aParam['scoring']) && (!isset($aGame['leaderboard_enabled']) || $aParam['scoring'] == 'yes' && !$aGame['leaderboard_enabled'] || $aParam['scoring'] == 'no' && $aGame['leaderboard_enabled']))
            {
                $bSkip = true;
            }
            elseif( isset($aParam['rating']) && (!isset($aGame['rating']) || !in_array(strtolower($aGame['rating']), $aParam['rating'])) )
            {
                $bSkip = true;
            }
            elseif( isset($aParam['tags']) && strlen($aParam['tags']) )
            {
                $aTags = explode(',', $aParam['tags']);

                foreach($aTags as $iTag => $sTag)
                {
                    $aTags[$iTag] = strtolower(trim($sTag));
                }

                $bTagged = false;
                if( isset($aGame['tags']) )
                {
                    foreach($aGame['tags'] as $iTag => $sTag)
                    {
                        if( in_array($sTag, $aTags) )
                        {
                            $bTagged = true;
                            break;
                        }
                    }
                }

                if( !$bTagged )
                {
                    $bSkip = true;
                }
            }

            if( isset($aParam['multiple_categories']) && $aParam['multiple_categories'] == 'yes' )
            {
                $aGame['category'] = $aGame['categories'];
            }
            else
            {
                $aGame['category'] = (array) $aGame['category'];
            }

            if( isset($aParam['category']) )
            {
                $bHasCategory = false;

                foreach($aGame['category'] as $iCategory => $sCategory)
                {
                    $sCategory = strtolower(trim($sCategory));

                    if( !isset($this->_aMap[$sCategory]) )
                    {
                        continue;
                    }

                    $iMap = $this->_aMap[$sCategory];

                    if( in_array($iMap, $aParam['category']) )
                    {
                        $bHasCategory = true;

                        break;
                    }
                }

                if( !$bHasCategory )
                {
                    $bSkip = true;
                }
            }

            if( $bSkip )
            {
                continue;
            }

            $aGames[] = $aGame;
        }

        return $aGames;
    }

    /**
     * Gets a single game from source
     *
     * @param string $sId
     * @return mixed
     */
    public function importSingle($sId)
    {
        $oFile = SAA_Sys::getLib('file');

        $sUrl = 'http://www.mochimedia.com/feeds/games/'. SAA_Sys::getSett('mochi.publisher_id') .'/'.$sId.'/?format=json';

        if( !$oFile->downloadRemote($sUrl, SAA_APP_CACHE . 'adder.tmp.txt') )
        {
            return SAA_Error::add('JSON file could not be downloaded to server.');
        }

        if( !($sRaw = file_get_contents(SAA_APP_CACHE . 'adder.tmp.txt')) )
        {
            return SAA_Error::add('JSON file could not be opened.');
        }

        $aJson = json_decode($sRaw, true);

        if( is_null($aJson) )
        {
            return SAA_Error::add('JSON file could not be accessed.');
        }
        elseif( !isset($aJson['games']) )
        {
            return SAA_Error::add('Games key could not be accessed.');
        }
		elseif( !count($aJson['games']) )
		{
			return SAA_Error::add('Game no longer exists in the feed.');
		}

        $oFile->delete(SAA_APP_CACHE . 'adder.tmp.txt');

        $aGame = $aJson['games'][0];

        $aGame['category'] = $aGame['categories'];

        return $aGame;
    }

    /**
     * Sets the maximum games
     *
     * @param int $iMax
     */
    public function setMax($iMax)
    {
        $this->_aParam['max'] = $iMax;
    }
}
?>
