<?php
defined('Trapper') or exit('Trespasser!');

/**
 *
 * @author Hutton
 * @name Language Phrase Process
 * @license GoLevel LLC
 */

Class SAA_Model_Language_Phrase_Process extends SAA_Model
{
    private $_aCols = array(
        'language_id',
        'section_id',
        'var_name',
        'text',
        'text_default',
        'time_added'
    );

    /**
     * Class Constructor
     */
    public function  __construct()
    {

    }

    /**
     * Add a phrase to all existing languages
     *
     * @param array $aVal array of text for the given phrase where the array keys are language ids
     * @param string $sSection the section_id
     * @return mixed
     */
    public function add($aVal)
    {
        if( !$this->verifyInput($aVal['text']) )
        {
            return false;
        }

        if( !strlen($aVal['section_id']) )
        {
            SAA_Error::add('Please choose a section.');
        }
        elseif( !($aSection = SAA_Sys::getModel('language.section')->get($aVal['section_id'], false)) )
        {
            SAA_Error::add('Section is not valid.');
        }

        if( !strlen($aVal['var_name']) )
        {
            SAA_Error::add('Please enter a phrase name.');
        }

        if( SAA_Error::get() )
        {
            return false;
        }

        $sVarName = $this->parse()->toAlphaNumeric($aVal['var_name'], 250, '_', '_');
        $sVarName = $this->parse()->getUnique($sVarName, 'language_phrase', array('var_name', 'section_id' => $aVal['section_id']), 250);

        foreach($aVal['text'] as $iLanguage => $sText)
        {
            $sText = $this->parse()->rareToEntities($sText);

            $aSql = array(
                'language_id' => $iLanguage,
                'section_id' => $aSection['section_name'],
                'var_name' => $sVarName,
                'text' => $sText,
                'text_default' => $sText,
                'time_added' => SAA_TIME
            );

            $iId = $this->database()->insert('language_phrase', $aSql);
        }

        if( !$iId )
        {
            return false;
        }

        $this->clearCache();

        return array(
            'section_id' => $aSql['section_id'],
            'var_name' => $aSql['var_name']
        );
    }

    /**
     * Adds any missing phrases to a given language using the default language
     *
     * @param int $iLanguage
     * @return bool
     */
    public function addMissing($iLanguage)
    {
        $iDefault = SAA_Sys::getLib('language')->getDefault();

        $aDefaultPhrases = $this->database()
            ->select('l.section_id, l.var_name, l.text, l.text_default')
            ->from('language_phrase', 'l')
            ->where('l.language_id = '. (int) $iDefault)
            ->get('rows');

        $aNewPhrases = $this->database()
            ->select('l.section_id, l.var_name, l.text, l.text_default')
            ->from('language_phrase', 'l')
            ->where('l.language_id = '. (int) $iLanguage)
            ->get('rows');

        $aPhrases = array();

        foreach($aDefaultPhrases as $aDefaultPhrase)
        {
            $bAdd = true;

            foreach($aNewPhrases as $aPhrase)
            {
                if( $aDefaultPhrase['var_name'] == $aPhrase['var_name'] && $aDefaultPhrase['section_id'] == $aPhrase['section_id'] )
                {
                    $bAdd = false;
                    break;
                }
            }

            if( $bAdd )
            {
                $aPhrases[] = $aDefaultPhrase;
            }
        }

        $aSql = array();

        foreach($aPhrases as $aPhrase)
        {
            $aSql[] = array(
                $iLanguage,
                $aPhrase['section_id'],
                $aPhrase['var_name'],
                $aPhrase['text'],
                $aPhrase['text_default'],
                SAA_TIME,
            );
        }

        if( count($aSql) )
        {
            $this->database()->insertMultiple('language_phrase', $this->_aCols, $aSql);
        }

        return true;
    }

    /**
     * Adds new phrases found in a given language to all other languages
     *
     * @param int $iLanguage
     * @return bool
     */
    public function addMissingToAll($iLanguage)
    {
        $aLanguages = SAA_Sys::getLib('language')->getLanguages();
        $iDefault = SAA_Sys::getLib('language')->getDefault();

        unset($aLanguages[$iLanguage]);

        $aDefaultPhrases = $this->database()
            ->select('l.section_id, l.var_name, l.text, l.text_default')
            ->from('language_phrase', 'l')
            ->where('l.language_id = '. (int) $iDefault)
            ->get('rows');

        $aNewPhrases = $this->database()
            ->select('l.section_id, l.var_name, l.text, l.text_default')
            ->from('language_phrase', 'l')
            ->where('l.language_id = '. (int) $iLanguage)
            ->get('rows');

        $aPhrases = array();

        foreach($aNewPhrases as $aPhrase)
        {
            $bAdd = true;

            foreach($aDefaultPhrases as $aDefaultPhrase)
            {
                if( $aPhrase['var_name'] == $aDefaultPhrase['var_name'] && $aPhrase['section_id'] == $aDefaultPhrase['section_id'] )
                {
                    $bAdd = false;
                    break;
                }
            }

            if( $bAdd )
            {
                $aPhrases[] = $aPhrase;
            }
        }

        $aSql = array();

        foreach($aPhrases as $aPhrase)
        {
            foreach($aLanguages as $iLanguage => $aLanguage)
            {
                $aSql[] = array(
                    $iLanguage,
                    $aPhrase['section_id'],
                    $aPhrase['var_name'],
                    $aPhrase['text'],
                    $aPhrase['text_default'],
                    SAA_TIME,
                );
            }
        }

        if( count($aSql) )
        {
            $this->database()->insertMultiple('language_phrase', $this->_aCols, $aSql);
        }

        return true;
    }

    /**
     * Removes all cache files for language phrases
     *
     * @return bool
     */
    public function clearCache()
    {
        $aLanguages = SAA_Sys::getLib('language')->getLanguages();

        foreach($aLanguages as $aLanguage)
        {
            $this->cache()->remove('language_phrases_' . $aLanguage['language_id']);
        }

        return true;
    }

    /**
     * Changes the data for a given phrase over a given languages
     *
     * @param array $aVals updated phrase data
     * @return bool
     */
    public function edit($aVal)
    {
        $aSql = array(
            'text' => $this->parse()->rareToEntities($aVal['text'])
        );

        $this->database()->update('language_phrase', $aSql, 'phrase_id = '. (int) $aVal['phrase_id']);

        $this->clearCache();

        return true;
    }

    /**
     * Changes the data for a given phrase over all languages
     *
     * @param array $aVals updated phrase data
     * @return bool
     */
    public function editAll($aVals, $sPhraseVar)
    {
        list($sSection, $sVar) = explode('.', $sPhraseVar);

        foreach($aVals as $iLanguage => $sText)
        {
            $aSql = array(
                'text' => $this->parse()->rareToEntities($sText)
            );

            $this->database()->update('language_phrase', $aSql, 'var_name = "'.$this->database()->esc($sVar).'" AND section_id = "'.$this->database()->esc($sSection).'" AND language_id = '. (int) $iLanguage);
        }

        $this->clearCache();

        return true;
    }

    /**
     * Removes a phrase from all languages
     *
     * @param mixed $mId
     * @param bool $bById
     * @return bool
     */
    public function delete($mId, $bById = false)
    {
        if( $bById )
        {
            $aPhrase = SAA_Sys::getModel('language.phrase')->get($mId);

            if( !$aPhrase )
            {
                return false;
            }

            $mId = $aPhrase['section_id'] . '.' . $aPhrase['var_name'];
        }

        $aPhrase = explode('.', $mId);

        $this->database()->delete('language_phrase', 'section_id = "'. $this->database()->esc($aPhrase[0]) .'" AND var_name = "'.$this->database()->esc($aPhrase[1]).'"');

        //clears the languages cache
        $this->clearCache();

        return true;
    }

    /**
     * Restores a phrase's text to default text
     *
     * @param int $iPhrase
     * @return bool
     */
    public function revert($iPhrase)
    {
        $aPhrase = SAA_Sys::getModel('language.phrase')->get($iPhrase);

        if( !$aPhrase )
        {
            return false;
        }

        $this->database()->update('language_phrase', array('text' => $aPhrase['text_default']), 'phrase_id = '. (int) $aPhrase['phrase_id']);

        $this->cache()->remove('language_phrases_' . $aPhrase['language_id']);

        return true;
    }

    /**
     * Checks the inputed values for no missing/extra languages and text exists
     *
     * @param array $aVals array of phrases with the array keys as the language id
     * @return bool
     */
    public function verifyInput($aVals)
    {
        $aLanguages = SAA_Sys::getLib('language')->getLanguages();

        foreach($aVals as $iLanguage => $sText)
        {
            if( !array_key_exists($iLanguage, $aLanguages) )
            {
                return SAA_Error::add('Language key is invalid.');
            }
            elseif( !strlen($sText) )
            {
                return SAA_Error::add('Please provide a name for all language phrases.');
            }
        }

        if( count($aLanguages) != count($aVals) )
        {
            return SAA_Error::add('Language key is missing.');
        }

	return true;
    }
}
?>
