<?php
defined('Trapper') or exit('Trespasser!');

/**
 *
 * @author Hutton
 * @name Database Abstraction Library Class
 * @license GoLevel LLC
 */

Abstract Class SAA_Database_Abstraction
{
    protected $_sPrefix;

    public $_bPrefix = true;

    private $_aQuery = array();

    /**
     * Class Constructor
     */
    public function __construct()
    {
        $this->_sPrefix = SAA_Sys::getSett('db.prefix');
    }

    /**
     * Removes a given set of rows from a table
     *
     * @param string $sTable
     * @param string $sWhere
     * @param int $iLimit
     * @return resource
     */
    public function delete($sTable, $sWhere, $iLimit = null)
    {
    	if ($iLimit !== null)
    	{
	    $sWhere .= ' LIMIT ' . (int) $iLimit;
    	}

    	return $this->query("DELETE FROM ". $this->table($sTable) ." WHERE ". $sWhere);
    }

    /**
     * Sets the from part of current queued database call
     *
     * @param string $sTable
     * @param string $sAlias
     * @return SAA_Database_Abstraction
     */
    public function from($sTable, $sAlias = '')
    {
	$this->_aQuery['from'] = 'FROM '. $this->table($sTable) . ($sAlias ? ' AS ' . $sAlias : '');

        return $this;
    }

    /**
     * Executes the current queued database call
     *
     * @param string $sType
     * @return array
     */
    public function get($sType = null)
    {
	$sSql = $this->_aQuery['select'] . "\n";

	$sSql .= $this->_aQuery['from'] . "\n";

	if( isset($this->_aQuery['join']) )
	{
	    $sSql .= $this->_aQuery['join'] . "\n";
	}

	if( isset($this->_aQuery['where']) )
	{
	    $sSql .= rtrim(trim($this->_aQuery['where']), 'AND') . "\n";
	}

	if( isset($this->_aQuery['group']) )
	{
	    $sSql .= $this->_aQuery['group'] . "\n";
	}

	if( isset($this->_aQuery['having']) )
	{
	    $sSql .= $this->_aQuery['having'] . "\n";
	}

	if( isset($this->_aQuery['order']) )
	{
	    $sSql .= $this->_aQuery['order'] . "\n";
	}

	if( isset($this->_aQuery['limit']) )
	{
	    $sSql .= $this->_aQuery['limit'] . "\n";
	}

	switch($sType)
	{
	    case 'row':
		$aRows = $this->getRow($sSql);
		break;
	    case 'rows':
                $aRows = $this->getRows($sSql);
		break;
	    case 'column':
		$aRows = $this->getField($sSql);
		break;
	    default:
		return $sSql;
		break;
	}

        $this->_aQuery = array();

	return $aRows;
    }

    /**
     * Gets the returned row for a provided query
     *
     * @param string $sSql
     * @param bool $bAssoc
     * @return array
     */
    public function getRow($sSql, $bAssoc = true)
    {
    	return $this->_getRow($sSql, $bAssoc);
    }

    /**
     * Gets the returned rows for a provided query
     *
     * @param string $sSql
     * @param bool $bAssoc
     * @return array
     */
    public function getRows($sSql, $bAssoc = true)
    {
    	return $this->_getRows($sSql, $bAssoc, (isset($this->_aQuery['key']) ? $this->_aQuery['key'] : null));
    }

    /**
     * Gets the returned column for a provided query
     *
     * @param string $sSql
     * @return string
     */
    public function getField($sSql)
    {
	return $this->_getField($sSql);
    }

    /**
     * Sets the group part of current queued database call
     *
     * @param string $sGroup
     * @return SAA_Database_Abstraction
     */
    public function group($sGroup)
    {
	$this->_aQuery['group'] = 'GROUP BY ' . $sGroup;

        return $this;
    }

    /**
     * Sets the having part of current queued database call
     *
     * @param string $sHaving
     * @return SAA_Database_Abstraction
     */
    public function having($sHaving)
    {
	$this->_aQuery['having'] = 'HAVING ' . $sHaving;

        return $this;
    }

    /**
     * Sets the column value that will be the key for the returned rows
     *
     * @param string $sColumn
     * @return SAA_Database_Abstraction
     */
    public function key($sColumn)
    {
        $this->_aQuery['key'] = $sColumn;

        return $this;
    }

    /**
     * Sets the join part of current queued database call
     *
     * @param string $sTable
     * @param string $sAlias
     * @param string $sConditions
     * @param string $sSide
     * @return SAA_Database_Abstraction
     */
    public function join($sTable, $sAlias, $sConditions, $sSide = '')
    {
	if( !isset($this->_aQuery['join']) )
	{
	    $this->_aQuery['join'] = '';
	}

	$this->_aQuery['join'] .= ' '.($sSide ? strtoupper($sSide) . ' ' : '') . 'JOIN '. $this->table($sTable) . ' AS ' . $sAlias . ' ' .( strlen($sConditions) ? 'ON '. $sConditions.' ' : '');

        return $this;
    }

    /**
     * Sets the left join part of current queued database call
     *
     * @param string $sTable
     * @param string $sAlias
     * @param string $sConditions
     * @return SAA_Database_Abstraction
     */
    public function leftJoin($sTable, $sAlias, $sConditions = '')
    {
	$this->join($sTable, $sAlias, $sConditions, $sSide = 'left');

        return $this;
    }

    /**
     * Sets the limit (with an option to offset via page number) part of current queued database call
     *
     * @param int $iLimit
     * @param mixed $mPage
     * @return SAA_Database_Abstraction
     */
    public function limit($iLimit, $mPage = 1)
    {
	$this->_aQuery['limit'] = 'LIMIT '. (int) $iLimit;

        if( $mPage > 1 )
        {
            $this->_aQuery['limit'] .= ' OFFSET ' . (($mPage-1)*$iLimit);
        }

        return $this;
    }

    /**
     * Sets the order part of current queued database call
     *
     * @param mixed $mColumn
     * @param string $sOrder
     * @return SAA_Database_Abstraction
     */
    public function order($mColumn, $sOrder = '')
    {
        if( is_array($mColumn) )
        {
            $this->_aQuery['order'] = 'ORDER BY ';

            foreach($mColumn as $sColumn => $sOrder)
            {
                $this->_aQuery['order'] .= $sColumn . ' ' . $sOrder . ', ';
            }

            $this->_aQuery['order'] = rtrim($this->_aQuery['order'], ', ');
        }
        else
        {
            $this->_aQuery['order'] = 'ORDER BY ' . $mColumn . ' ' . ($sOrder ? $sOrder : '');
        }

        return $this;
    }

    /**
     * Sets the right join part of current queued database call
     *
     * @param string $sTable
     * @param string $sAlias
     * @param string $sConditions
     * @return SAA_Database_Abstraction
     */
    public function rightJoin($sTable, $sAlias, $sConditions = '')
    {
	$this->join($sTable, $sAlias, $sConditions, $sSide = 'right');

        return $this;
    }

    /**
     * Sets/appends the select part of current queued database call
     *
     * @param string $sSelect
     * @return SAA_Database_Abstraction
     */
    public function select($sSelect)
    {
	if( !isset($this->_aQuery['select']) )
	{
	    $this->_aQuery['select'] = 'SELECT ';
	}

	$this->_aQuery['select'] .= $sSelect;

        return $this;
    }

    /**
     * Gets a table name
     *
     * @param string $sTable
     * @return string
     */
    public function table($sTable)
    {
        return ($this->_bPrefix ? $this->_prefix($sTable) : $sTable);
    }

    /**
     * Turns table prefix on/off
     *
     * @param bool $bSwitchOn
     * @return SAA_Database_Abstraction
     */
    public function togglePrefix($bSwitchOn = false)
    {
        $this->_bPrefix = ($bSwitchOn ? true : false);

        return $this;
    }

    /**
     * Sets the where part of current queued database call
     *
     * @param mixed $mWhere
     * @return SAA_Database_Abstraction
     */
    public function where($mWhere)
    {
        if( !isset($this->_aQuery['where']) )
	{
	    $this->_aQuery['where'] = 'WHERE ';
	}

	if( is_string($mWhere) )
	{
	    $mWhere = (array) $mWhere;
	}

	foreach($mWhere as $sWhere)
	{
	    $this->_aQuery['where'] .= ' ' . $sWhere;
	}

        $this->_aQuery['where'] = trim($this->_aQuery['where']);
        $this->_aQuery['where'] = trim($this->_aQuery['where'], ' AND');
        $this->_aQuery['where'] = trim($this->_aQuery['where'], ' and');
        $this->_aQuery['where'] = trim($this->_aQuery['where'], ' or');
        $this->_aQuery['where'] = trim($this->_aQuery['where'], ' OR');

        return $this;
    }

    /**
     * Prepends a table with the prefix
     *
     * @param string $sTable
     * @return string
     */
    protected function _prefix($sTable)
    {
        return $this->_sPrefix . $sTable;
    }
}
?>