<?php
defined('Trapper') or exit('Trespasser!');

/**
 *
 * @author Hutton
 * @name Language Library Class
 * @license GoLevel LLC
 */

Class SAA_Language
{
    private $_aLanguages = array();
    private $_aPhrases = array();
    private $_iDefault;

     /**
     * Class Constructor
     *
     * Gets the list of languages and phrases for the current phrase
     */
    public function  __construct()
    {
        $oCache = SAA_Sys::getLib('cache');

        if( ($aLanguages = $oCache->get('languages')) )
        {
            $this->_aLanguages = $aLanguages;
        }
        else
        {
            $aLanguages = SAA_Sys::getModel('language')->getAll();

            $this->_getLanguages($aLanguages);

            $oCache->create('languages', $this->_aLanguages);
        }

        $this->_setDefault();

        $sCache = 'language_phrases_' . $this->_iDefault;
        if( !($aPhrases = $oCache->get($sCache)) )
        {
            $aPhrases = SAA_Sys::getModel('language.phrase')->getAllById($this->_iDefault);
            $this->_getPhrases($aPhrases);

            $aPhrases = $this->_aPhrases;

            $oCache->create($sCache, $this->_aPhrases);
        }

        $this->_aPhrases = $aPhrases;
    }

    /**
     * Gets the default language id or array
     * 
     * @return bool $bId
     * @return mixed
     */
    public function getDefault($bId = true)
    {
        return ($bId ? $this->_aLanguages[$this->_iDefault]['language_id'] : $this->_aLanguages[$this->_iDefault]);
    }

    /**
     * Get a given language by id
     * 
     * @param int $iLanguage
     * @return mixed
     */
    public function getLanguage($iLanguage)
    {
        return (isset($this->_aLanguages[$iLanguage]) ? $this->_aLanguages[$iLanguage] : false);
    }

    /**
     * Get all languages
     *
     * @param bool $bPublic
     * @return array
     */
    public function getLanguages($bPublic = false)
    {
        if( !$bPublic )
        {
            return $this->_aLanguages;
        }

        $aLanguages = array();

        foreach($this->_aLanguages as $aLanguage)
        {
            if( $aLanguage['is_enabled'] )
            {
                $aLanguages[] = $aLanguage;
            }
        }

        return $aLanguages;
    }

    /**
     * Gets a given phrase
     *
     * @param string $sPhrase phrase var
     * @param array $aVars parameters
     * @return mixed
     */
    public function getPhrase($sPhrase, $aVars = array())
    {
        if( !$this->isPhrase($sPhrase) )
        {
            return SAA_Debug::trigger('The phrase "'.$sPhrase.'" does not exist.');
        }

        $sText = $this->_aPhrases[$sPhrase];

        if( count($aVars) )
        {
            $aFind = array();
            $aReplace = array();

            foreach($aVars as $sSearch => $sReplace)
            {
                $aFind[] = '{'.$sSearch.'}';
                $aReplace[] = $sReplace;
            }

            if( count($aFind) )
            {
                $sText = str_replace($aFind, $aReplace, $sText);
            }
        }

        if( isset($aVars['js_escape']) )
        {
            $sText = str_replace("'", "\\'", $sText);
        }

        return $sText;
    }

    /**
     * Checks if a phrase is valid
     *
     * @param string $sVarName phrase var
     * @return bool
     */
    public function isPhrase($sVarName)
    {
        return (isset($this->_aPhrases[$sVarName]) ? true : false);
    }

    /**
     * Adds a phrase to phrases
     *
     * @param string $sVarName
     * @param string $sValue
     * @return bool
     */
    public function setPhrase($sVarName, $sValue)
    {
        $this->_aPhrases[$sVarName] = $sValue;

        return true;
    }

    /**
     * Set a language as default
     *
     * @param int $iId
     * @return bool
     */
    public function setDefault($iId)
    {
        if( !isset($this->getLanguage[$iId]) )
        {
            return false;
        }

        $this->_iDefault = $iId;

        return true;
    }

    /**
     * Builds language array
     *
     * @param array $aLanguages
     */
    private function _getLanguages($aLanguages)
    {
        foreach($aLanguages as $aLanguage)
        {
            $this->_aLanguages[$aLanguage['language_id']] = $aLanguage;
        }

    }

    /**
     * Builds phrase array
     *
     * @param array $aPhrases
     */
    private function _getPhrases($aPhrases)
    {
        foreach($aPhrases as $aPhrase)
        {
            $sVarName = $aPhrase['section_id'] . '.' . $aPhrase['var_name'];
            $this->_aPhrases[$sVarName] = $aPhrase['text'];
        }
    }

    /**
     * Determines the default language
     *
     * @return true
     */
    private function _setDefault()
    {
        //use user's data
        if( SAA_Sys::isUser() )
        {
            $iId = SAA_Sys::getModel('user.session')->get('lang_id');

            if( isset($this->_aLanguages[$iId]) && $this->_aLanguages[$iId]['is_enabled'] )
            {
                $this->_iDefault = $iId;

                return true;
            }
        }

        foreach($this->_aLanguages as $aLanguage)
        {
            if( $aLanguage['is_default'] )
            {
                $this->_iDefault = $aLanguage['language_id'];
                
                break;
            }
        }
    }
}
?>
